/* Copyright (C) 2003 Stephane Thiell
 *
 * This file is part of pxyservd (from pxys)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#define RCSID "$Id: PXConfig.cc,v 1.1.1.1 2003/12/30 17:09:00 mbuna Exp $"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "PXConfig.h"

#include <cstring>

PXConfig::PXConfig()
  {
  this->opas.local_address.s_addr = INADDR_ANY;
  }

PXConfig::~PXConfig()
  {
  }

bool
PXConfig::OPASIsAllowed(const in_addr &in) const
  {
  for (size_t i = 0; i < opas.allow.size(); i++)
    if (opas.allow[i].s_addr == in.s_addr)
      return true;
  return false;
  }

in_addr
PXConfig::GetOneFromSourcePool() const
  {
  assert(scanner.source_pool.size() > 0);
  
  StPXMutex m(&mutex);
  return scanner.source_pool[(int)((double(rand())
                                    / double((unsigned long)RAND_MAX + 1))
                                   * scanner.source_pool.size())];
  }

bool
PXConfig::ScannerStringCheck(const char *s) const
  {
  for (size_t i = 0; i < scanner.strings.size(); i++)
    if (strstr(s, scanner.strings[i]))
      return true;
  return false;
  }

bool
PXConfig::CheckNoScan(const in_addr &addr) const
  {
  for (size_t i = 0; i < this->noscan.address.size(); i++)
    {
    const NetworkNetmask * nn = &this->noscan.address[i];
    if ((addr.s_addr & nn->netmask.s_addr)
        == (nn->network.s_addr & nn->netmask.s_addr))
      return true;
    }
  return false;
  }
